/**
 * Build system.
 * @module gulpfile
 */
'use strict';

// Module dependencies.
var child=require('child_process');
var del=require('del');
var fs=require('fs');
var gulp=require('gulp');
var path=require('path');
var plugins=require('gulp-load-plugins')();
var pkg=require('./package.json');
var request=require('superagent');
var util=require('util');

/**
 * Provides tasks for [Gulp.js](http://gulpjs.com) build system.
 * @class cli.Gulpfile
 * @static
 */

/**
 * The task settings.
 * @property config
 * @type Object
 */
var config={
  dartsdk: util.format('https://storage.googleapis.com/dart-archive/channels/stable/release/latest/sdk/dartsdk-%s-x64-release.zip', process.platform=='win32' ? 'windows' : 'linux'),
  output: util.format('%s-%s.zip', pkg.yuidoc.name.toLowerCase(), pkg.version)
};

/**
 * Runs the default tasks.
 * @method default
 */
gulp.task('default', [ 'js' ]);

/**
 * Checks the package dependencies.
 * @method check
 */
gulp.task('check', function() {
  return gulp.src('package.json')
    .pipe(plugins.david())
    .pipe(plugins.david.reporter);
});

/**
 * Deletes all generated files and reset any saved state.
 * @method clean
 */
gulp.task('clean', function(callback) {
  var sources=[
    '**/packages',
    'var/dart-sdk',
    'var/dart-sdk.zip',
    'var/'+config.output
  ];

  del(sources, callback);
});

/**
 * Installs a local Dart SDK if required.
 * @method dart
 */
gulp.task('dart', [ 'dart:chmod' ], function(callback) {
  process.env.DART_SDK=path.resolve('var/dart-sdk');
  _exec(util.format('"%s" get', path.join(process.env.DART_SDK, 'bin/pub')), callback);
});

gulp.task('dart:chmod', [ 'dart:unzip' ], function(callback) {
  if(process.platform=='win32') callback();
  else {
    var basePath='var/dart-sdk/bin';
    fs.readdir(basePath, function(err, files) {
      if(err) callback(err);

      files.forEach(function(file) {
        fs.chmodSync(path.join(basePath, file), parseInt('755', 8));
      });

      callback();
    });
  }
});

gulp.task('dart:sdk', function(callback) {
  request.get(config.dartsdk)
    .on('end', callback)
    .on('error', callback)
    .pipe(fs.createWriteStream('var/dart-sdk.zip'));
});

gulp.task('dart:unzip', [ 'dart:sdk' ], function() {
  return gulp.src('var/dart-sdk.zip')
    .pipe(plugins.unzip())
    .pipe(gulp.dest('var'));
});

/**
 * Creates a distribution file for this program.
 * @method dist
 */
gulp.task('dist', [ 'default' ], function() {
  var sources=[
    'pubspec.yaml',
    '*.md',
    '*.txt',
    'bin/*',
    'lib/**/*.dart',
    'test/**/*.dart',
    '!**/packages'
  ];

  return gulp.src(sources, { base: '.' })
    .pipe(plugins.zip(config.output))
    .pipe(gulp.dest('var'));
});

/**
 * Builds the documentation.
 * @method doc
 */
gulp.task('doc', [ 'doc:assets' ]);

gulp.task('doc:assets', [ 'doc:build' ], function() {
  return gulp.src([ 'www/apple-touch-icon.png', 'www/favicon.ico' ])
    .pipe(gulp.dest('doc/api/assets'));
});

gulp.task('doc:build', function(callback) {
  _exec('docgen', callback);
});

/**
 * Builds the client scripts.
 * @method js
 */
gulp.task('js', [ 'js:clean' ]);

gulp.task('js:build', [ 'dart' ], function(callback) {
  var command=util.format(
    '"%s" --minify --out="%s" "%s"',
    path.join(process.env.DART_SDK, 'bin/dart2js'),
    path.resolve('www/js/tests.js'),
    path.resolve('test/html_test.dart')
  );

  _exec(command, callback);
});

gulp.task('js:clean', [ 'js:build' ], function(callback) {
  del([ 'www/js/*.js.deps' ], callback);
});

/**
 * Performs static analysis of source code.
 * @method lint
 */
gulp.task('lint', [ 'lint:dart', 'lint:doc', 'lint:js' ]);

gulp.task('lint:dart', function(callback) {
  var command=util.format(
    '"%s" --package-root="%s" "%s" "%s"',
    path.join(process.env.DART_SDK, 'bin/dartanalyzer'),
    path.resolve('packages'),
    path.resolve('lib/html.dart'),
    path.resolve('lib/io.dart')
  );

  _exec(command, callback);
});

gulp.task('lint:doc', function(callback) {
  _exec('docgen --lint', callback);
});

gulp.task('lint:js', function() {
  return gulp.src('*.js')
    .pipe(plugins.jshint(pkg.jshintConfig))
    .pipe(plugins.jshint.reporter('default', { verbose: true }));
});

/**
 * Starts the Web server.
 * @method serve
 */
gulp.task('serve', [ 'dart' ], function(callback) {
  if('_server' in config) {
    config._server.kill();
    delete config._server;
  }

  config._server=child.spawn(path.join(process.env.DART_SDK, 'bin/dart'), [ path.resolve('bin/cli.dart') ]);
  callback();
});

/**
 * Runs the unit tests.
 * @method test
 */
gulp.task('test', [ 'dart', 'test:env' ], function(callback) {
  var command=util.format(
    '"%s" "%s"',
    path.join(process.env.DART_SDK, 'bin/dart'),
    path.resolve('test/io_test.dart')
  );

  _exec(command, callback);
});

gulp.task('test:env', function(callback) {
  if('AKISMET_API_KEY' in process.env) callback();
  else callback(new Error('AKISMET_API_KEY environment variable not set.'));
});

/**
 * Watches for file changes.
 * @method watch
 */
gulp.task('watch', [ 'js', 'serve' ], function() {
  gulp.watch('lib/*.dart', [ 'js', 'serve' ]);
  gulp.watch([ 'test/*.dart', 'www/js/main.js' ], [ 'js' ]);
});

/**
 * Runs a command and prints its output.
 * @method _exec
 * @param {String} command The command to run, with space-separated arguments.
 * @param {Function} callback The function to invoke when the task is over.
 * @async
 * @private
 */
function _exec(command, callback) {
  child.exec(command, function(err, stdout) {
    var output=stdout.trim();
    if(output.length) console.log(output);
    if(err) console.error(err);
    callback();
  });
}
